/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.QueryView;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;

import ca.uvic.csr.shrimp.SelectorBeanConstants;
import ca.uvic.csr.shrimp.DisplayBean.ShrimpNode;
import ca.uvic.csr.shrimp.SelectorBean.SelectorBean;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectAdapter;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectEvent;
import ca.uvic.csr.shrimp.ShrimpTool.ShrimpTool;
import ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpView;
import ca.uvic.csr.shrimp.util.CollectionUtils;


/**
 * Links the selections in the {@link ShrimpView} with the search panel in the {@link QueryView}. 
 * 
 * @author Chris Callendar
 * @date 31-Jul-06
 */
public class ShrimpViewQueryViewBridge extends ShrimpProjectAdapter implements PropertyChangeListener {

	private QueryView queryView;

	public ShrimpViewQueryViewBridge(QueryView queryView) {
		this.queryView = queryView;
	}
	
	public void init() {
		boolean showRadioButtons = false;
		try {
			ShrimpProject project = queryView.getProject();
			if (project != null) {
				project.addProjectListener(this);
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW); 
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectorBean.addPropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, this);
				setSelectionToQueryView(selectorBean);
				showRadioButtons = true;
			}
		} catch (Exception e) {
		}
		
		QueryPanelComponent qpc = queryView.getQueryPanel();
		if (qpc != null) {
			qpc.getUseSelectedRadioButton().setVisible(showRadioButtons);
			qpc.getSearchByNameRadioButton().setVisible(showRadioButtons);
		}
	}
	
	public void projectClosing(ShrimpProjectEvent event) {
		removeListeners();
	}

	public void removeListeners() {
		try {
			ShrimpProject project = queryView.getProject();
			if (project != null) {
				project.removeProjectListener(this);
				ShrimpView shrimpView = (ShrimpView) project.getTool(ShrimpProject.SHRIMP_VIEW); 
				SelectorBean selectorBean = (SelectorBean) shrimpView.getBean(ShrimpTool.SELECTOR_BEAN);
				selectorBean.removePropertyChangeListener(SelectorBeanConstants.SELECTED_NODES, this);
			}
		} catch (Exception e) {
		}
	}

	/**
	 * Listens for selection changes and passes the selected nodes on to the {@link QueryView}.
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		Vector oldSelected = (Vector) evt.getOldValue();
		Vector newSelected = (Vector) evt.getNewValue();
		if (!CollectionUtils.haveSameElements(oldSelected, newSelected)) {
			setSelectionToQueryView(newSelected);
		}
	}

	/**
	 * Gets the selected nodes from {@link SelectorBean} and passes them on to the
	 * {@link QueryView}.
	 * @param selectorBean
	 */
	public void setSelectionToQueryView(SelectorBean selectorBean) {
		if ((queryView != null) && (selectorBean != null)) {
			Vector selectedNodes = (Vector) selectorBean.getSelected(SelectorBeanConstants.SELECTED_NODES);
			setSelectionToQueryView(selectedNodes);
		}
	}

	/**
	 * Gets the artifacts from the selected nodes and passes them on to the {@link QueryView}.
	 * @param selectedNodes
	 */	
	private void setSelectionToQueryView(Vector selectedNodes) {
		// get the artifacts from the nodes
		ArrayList selectedArtifacts = new ArrayList(selectedNodes.size());
		for (Iterator iter = selectedNodes.iterator(); iter.hasNext();) {
			ShrimpNode node = (ShrimpNode) iter.next();
			if (!selectedArtifacts.contains(node.getArtifact())) {
				selectedArtifacts.add(node.getArtifact());
			}
		}
		queryView.setSrcArtifacts(selectedArtifacts);
	}

	public void dispose() {
		if (queryView != null) {
			removeListeners();
			queryView = null;
		}
	}

}
